/*-----------------------------------------------------------------------------
 * DISCLAIMER.
 * This is a script for demonstrating the SELECT... FOR XML... solution
 * for generating Nested XML from a data base. This script is intended to
 * demonstrate some programming principles and no warranty is either 
 * expressed or implied.
 *
 *                   RUN THIS SCRIPT AT YOUR OWN RISK
 *
 * If you have any questions or comments, please contact me at:
 * http://www.tomasvera.com
 *---------------------------------------------------------------------------*/
 
/*  Create the Customers table */
CREATE TABLE [dbo].[Customers](
	[CustomerId] [bigint] IDENTITY(1,1) NOT NULL,
	[LastName] [varchar](80) NULL,
	[FirstName] [varchar](80) NULL,
	[Address1] [varchar](80) NULL,
	[Address2] [varchar](80) NULL,
	[City] [varchar](50) NULL,
	[State] [varchar](2) NULL,
	[PostalCode] [varchar](10) NULL,
 CONSTRAINT [PK_Customers] PRIMARY KEY CLUSTERED 
(
	[CustomerId] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO
/* Create the Orders table */
CREATE TABLE [dbo].[Orders](
	[OrderId] [bigint] IDENTITY(1,1) NOT NULL,
	[CustomerId] [bigint] NOT NULL,
	[DTCreated] [datetime] NULL,
	[Subtotal] [money] NULL,
	[Tax] [money] NULL,
	[Shipping] [money] NULL,
	[Total] [money] NULL
) ON [PRIMARY]

GO

ALTER TABLE [dbo].[Orders] ADD  CONSTRAINT [DF_Orders_DTCreated]  DEFAULT (getdate()) FOR [DTCreated]
GO

/* Create the OrderItems table */
CREATE TABLE [dbo].[OrderItems](
	[LineItemId] [bigint] IDENTITY(1,1) NOT NULL,
	[OrderId] [bigint] NULL,
	[SKU] [varchar](20) NULL,
	[ItemPrice] [money] NULL,
	[Quantity] [int] NULL,
	[ExtPrice] [money] NULL
) ON [PRIMARY]

GO

/*  Create the CustNotes table */
CREATE TABLE [dbo].[CustNotes](
	[CustNoteId] [bigint] IDENTITY(1,1) NOT NULL,
	[CustomerId] [bigint] NULL,
	[DTCreated] [datetime] NULL,
	[NoteText] [text] NULL
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]

GO

ALTER TABLE [dbo].[CustNotes] ADD  CONSTRAINT [DF_CustNotes_DTCreated]  DEFAULT (getdate()) FOR [DTCreated]
GO

/*-----------------------------------------------------------------------------
 * Data population section. We need data to demo things
 *----------------------------------------------------------------------------*/
-- Add some customers
INSERT INTO dbo.Customers ([FirstName], [LastName], [Address1], [City], [State], [PostalCode])
VALUES ('ANDY', 'ANDREWS', '123 Main St', 'ANYTOWN', 'AL', '99999')
GO
INSERT INTO dbo.Customers ([FirstName], [LastName], [Address1], [City], [State], [PostalCode])
VALUES ('BETTY', 'BOOP', '456 Pine St', 'HOMETWON', 'AK', '99999')
GO
INSERT INTO dbo.Customers ([FirstName], [LastName], [Address1], [City], [State], [PostalCode])
VALUES ('CHARLIE', 'COOL', '789 Scond Ave', 'NOWHERE', 'AR', '99999')
GO

-- Create some data for Andy
DECLARE @CustomerId bigint
DECLARE @OrderId bigint

-- Get Andy's CustomerID
SELECT @CustomerID = CustomerID from Customers where LastName='ANDREWS'
-- Create an Empty Order item and get its ID
INSERT INTO Orders (CustomerId,  DTCreated) VALUES(@CustomerId, GetDate())
SELECT @OrderID = SCOPE_IDENTITY()
-- Create some line items for Andy's Order
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, '123ABC', 1.99, 1 )
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, '456DEF', 2.99, 2)
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, '789GHI', 3.99, 3)

-- Create an Empty Order item and get its ID
INSERT INTO Orders (CustomerId,  DTCreated) VALUES(@CustomerId, GetDate())
SELECT @OrderID = SCOPE_IDENTITY()
-- Create some line items for Andy's Order
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, '123ABC', 4.99, 3 )
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, '456DEF', 5.99, 2)
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, '789GHI', 6.99, 1)

-- Insert some customer notes
INSERT INTO CustNotes (CustomerId, NoteText)
VALUES (@CustomerId, 'This is a note for customer Andy')
INSERT INTO CustNotes (CustomerId, NoteText)
VALUES (@CustomerId, 'This is a second note for customer Andy')


-- Get Boop's CustomerID
SELECT @CustomerID = CustomerID from Customers where LastName='BOOP'
-- Create an Empty Order item and get its ID
INSERT INTO Orders (CustomerId,  DTCreated) VALUES(@CustomerId, GetDate())
SELECT @OrderID = SCOPE_IDENTITY()
-- Create some line items for Andy's Order
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'a1a1a1', 7.99, 1 )
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'b2b2b2', 8.99, 2)
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'c3c3c3', 9.99, 3)

-- Create an Empty Order item and get its ID
INSERT INTO Orders (CustomerId,  DTCreated) VALUES(@CustomerId, GetDate())
SELECT @OrderID = SCOPE_IDENTITY()
-- Create some line items for Andy's Order
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'a99z', 1.99, 3 )
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, '1985G', 9.99, 2)
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, '711211', 2.99, 1)

-- Insert some customer notes
INSERT INTO CustNotes (CustomerId, NoteText)
VALUES (@CustomerId, 'This is a note for customer BOOP')
INSERT INTO CustNotes (CustomerId, NoteText)
VALUES (@CustomerId, 'This is a second note for customer BOOP')
INSERT INTO CustNotes (CustomerId, NoteText)
VALUES (@CustomerId, 'BOOP Gets 3 notes')

-- Get Andy's CustomerID
SELECT @CustomerID = CustomerID from Customers where LastName='COOL'
-- Create an Empty Order item and get its ID
INSERT INTO Orders (CustomerId,  DTCreated) VALUES(@CustomerId, GetDate())
SELECT @OrderID = SCOPE_IDENTITY()
-- Create some line items for Andy's Order
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'aloha1', 8.99, 1 )
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'Adios2', 3.99, 2)
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'ciao3', 7.99, 3)

-- Create an Empty Order item and get its ID
INSERT INTO Orders (CustomerId,  DTCreated) VALUES(@CustomerId, GetDate())
SELECT @OrderID = SCOPE_IDENTITY()
-- Create some line items for Andy's Order
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'yesyes', 4.99, 3 )
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'nonono', 6.99, 2)
INSERT INTO OrderItems (OrderId,  SKU,  ItemPrice, Quantity)
VALUES(@OrderId, 'alrighty', 5.99, 1)

-- Insert some customer notes
INSERT INTO CustNotes (CustomerId, NoteText)
VALUES (@CustomerId, 'COOL gets one note')


GO
-- Update the EXT Price
Update OrderItems set ExtPrice = ItemPrice * Quantity
GO
-- Update the order total tax, etc,
UPDATE Orders 
Set Subtotal = (SELECT SUM(ExtPrice) FROM ORderItems oi where oi.OrderID = orders.OrderId), 
	Tax= (SELECT SUM(ExtPrice) FROM ORderItems oi where oi.OrderID = orders.OrderId) * 0.05, 
	Shipping= (Select count(1) from OrderItems oi where oi.OrderID = orders.OrderId) * 1.25
	
Update Orders SET TOTAL = Subtotal + Tax + Shipping	
GO

/*-----------------------------------------------------------------------------
 *  END OF DATA POPULATION
 *----------------------------------------------------------------------------*/
 
/*-----------------------------------------------------------------------------
 *  Create a stored Procedure for use by C# Application sample
 *----------------------------------------------------------------------------*/
 CREATE PROCEDURE [dbo].[sp_GetDataForXml]
 AS
 BEGIN
	-- This just gets the data from the data table. Use a SqlDataAdapter
	-- to Fill() in a DataSet object in your C# code.
	SELECT * from Customers
	SELECT * from Orders
	SELECT * from OrderItems
	SELECT * From CustNotes
 END
 GO
/*----------------------------------------------------------------------------*/ 
--
-- if you have any questions or comments, please contact me at:
-- http://www.tomasvera.com
--